import theater.*;


public class AdvancedActor extends Actor {
    private double col;
    private double row;

    AdvancedActor() {
        this.col = 0.0;
        this.row = 0.0;
    }

    /**
     * berladene setLocation-Methode mit "genaueren" double-Werten
     */
    public void setLocation(double c, double r) {
        this.col = c;
        this.row = r;
        super.setLocation((int) c, (int) r);
    }

    /**
     * berschriebene setLocation-Methoden, um die internen col- und
     * row-Koordinaten zu setzen
     */
    public void setLocation(int col, int row) {
        this.setLocation((double) col, (double) row);
    }

    /**
     * liefert die przise col-Koordinate
     */
    public double getExactColumn() {
        return this.col;
    }

    /**
     * liefert die przise row-Koordinate
     */
    public double getExactRow() {
        return row;
    }

    /**
     * verschiebt den Akteur um einen bestimmten Rotationswinkel um einen bestimmten Punkt
     * (0 Grad liegt links vom Punkt)
     */
    public void setLocation(double degrees, double col, double row) {
        degrees = ((degrees + 180) % 360) - 180;

        // hack
        if (Math.abs(degrees) == 180) {
            setLocation(179.9, col, row);

            return;
        } else {
            double colDistance = getExactColumn() - col;
            double rowDistance = getExactRow() - row;
            double distance = Math.sqrt(((colDistance * colDistance) +
                    (rowDistance * rowDistance)));

            // hack
            this.col = col - distance;
            this.row = row;

            double theta = (180 - degrees) / 2;
            double c = (distance * sin(degrees)) / (sin(theta));
            double dx = c * cos(theta);
            double dy = c * sin(theta);

            setLocation(this.getExactColumn() + dx, this.getExactRow() - dy);
        }
    }

    /**
     * liefert die Rotation in Grad um einen bestimmten Punkte der Bhne
     * (0 Grad liegt links vom Punkt)
     */
    public double getRotation(double col, double row) {
        double colDistance = this.getExactColumn() - col;
        double rowDistance = this.getExactRow() - row;
        double angle = Math.toDegrees(Math.atan2(rowDistance, colDistance)) +
            180;

        return angle;
    }

    /**
     * bewegt den Akteur mit einer angegebenen Geschwindigkeit weiter in
     * Richtung seiner aktuellen Blickrichtung
     */
    public void move(double speed) {
        move(getRotation(), speed);
    }

    /**
     * bewegt den Akteur mit einer angegebenen Geschwindigkeit weiter in
     * eine bestimmte Richtung
     */
    public void move(int direction, double speed) {
        double angle = Math.toRadians(direction);
        int oldCol = getColumn();
        int oldRow = getRow();
        int newCol = (int) Math.round(oldCol + (Math.cos(angle) * speed));
        int newRow = (int) Math.round(oldRow + (Math.sin(angle) * speed));
        this.setLocation(newCol, newRow);
    }

    /**
     * lsst den Akteur um einen angegebenen Punkt kreisen; der Akteur schaut
     * dabei immer zum Punkt
     */
    public void circle(int col, int row) {
        while (true) {
            setLocation(getRotation(col, row) + 1, col, row);
            setRotation((int) getRotation(col, row));
        }
    }

    /**
     * bewegt den Akteur mit einer bestimmten Geschwindigkeit zu einer bestimmten Komponente
     */
    public void moveTo(Component comp, double speed) {
        while (!this.intersects(comp)) {
            int col = comp.getColumn();
            int row = comp.getRow();
            setRotation((int) getRotation(col, row));
            move(speed);
        }
    }

    private static double sin(double a) {
        return Math.sin(Math.toRadians(a));
    }

    private static double cos(double a) {
        return Math.cos(Math.toRadians(a));
    }

    private static double tan(double a) {
        return Math.tan(Math.toRadians(a));
    }
}
